@@ -298,8 +298,9 @@ data TypeSpecifier
   | Struct CIdentifier
   | Enum CIdentifier
   | TypeName CIdentifier
-  | Template CIdentifier [TypeSpecifier]
+  | Template CIdentifier [[TypeSpecifier]]
   | TemplateConst String
+  | TemplatePointer TypeSpecifier
   deriving (Typeable, Eq, Show)
 
 type_specifier :: CParser i m => m TypeSpecifier
@@ -379,14 +380,14 @@ templateParser s = parse'
     cidentParserWithNamespace =
       try (concat <$> sequence [cidentParser, (string "::"), cidentParserWithNamespace]) <|>
       cidentParser
-    templateArgType = try type_specifier <|> (TemplateConst <$> (many $ oneOf ['0'..'9']))
+    templateArgType = try ((TemplatePointer <$> (type_specifier)) <* (string "*")) <|> try type_specifier <|> (TemplateConst <$> (some $ oneOf ['0'..'9']))
     templateArgParser' = do
-      t <- templateArgType
+      t <- some (token templateArgType)
       _ <- string ","
       tt <- templateArgParser
       return $ t:tt
     templateArgParser =
-      try (templateArgParser') <|> ((:) <$> templateArgType <*> return [])
+      try (templateArgParser') <|> ((:) <$> some (token templateArgType) <*> return [])
 
 template_parser :: CParser i m => m TypeSpecifier
 template_parser = try $ templateParser cIdentStyle <?> "template name"
@@ -562,8 +563,13 @@ instance Pretty TypeSpecifier where
    Struct x -> "struct" <+> pretty x
    Enum x -> "enum" <+> pretty x
    TypeName x -> pretty x
-   Template x args -> pretty x <+> "<" <+> mconcat (intersperse "," (map pretty args))  <+> ">"
+   Template x args ->
+     -- This code generates a c++ code of "template-identifier<template-argument1,template-argument2,..>" like "std::vector<int>".
+     -- concat_with_space is used to concat multiple terms like "unsigned int".
+     let concat_with_space = mconcat . (intersperse " ") . (map pretty)
+     in pretty x <+> "<" <+> mconcat (intersperse "," (map concat_with_space args))  <+> ">"
    TemplateConst x -> pretty x
+   TemplatePointer x -> pretty x <+> "*"
 
 instance Pretty TypeQualifier where
   pretty tyQual = case tyQual of
